import { cleanup, render } from "@testing-library/react";
import CodeMirror from "codemirror";
import React from "react";

import { CodeEditor, CodeEditorMode, CodeEditorProps } from ".";

type RenderComponentFn = (
  props?: Partial<CodeEditorProps>
) => ReturnType<typeof render>;

describe("CodeEditor", () => {
  let renderComponent: RenderComponentFn;
  let defaultMode: CodeEditorMode;
  let defaultOnChange: any;
  let fromTextAreaSpy: any;

  beforeAll(() => {
    (document.body as any).createTextRange = jest.fn(() => ({
      setEnd() {},
      setStart() {},
      getBoundingClientRect() {
        return { right: 0 };
      },
      getClientRects() {
        return {
          length: 0,
          left: 0,
          right: 0
        };
      }
    }));
    defaultMode = CodeEditorMode.JavaScript;
    defaultOnChange = () => {};
    renderComponent = (additionalProps = {}) =>
      render(
        <CodeEditor
          mode={defaultMode}
          onChange={defaultOnChange}
          {...additionalProps}
        />
      );
  });

  beforeEach(() => {
    fromTextAreaSpy = jest.spyOn(CodeMirror, "fromTextArea");
  });

  afterEach(() => {
    fromTextAreaSpy.mockClear();
    cleanup();
  });

  it("fromTextArea is called properly with correct default arguments", () => {
    const { getByTestId } = renderComponent();

    const textarea = getByTestId("textarea");
    expect(fromTextAreaSpy).toHaveBeenCalledTimes(1);
    expect(fromTextAreaSpy).toHaveBeenCalledWith(textarea, {
      mode: defaultMode,
      lineNumbers: false
    });
  });

  it("fromTextArea is called properly with passed arguments", () => {
    const { getByTestId } = renderComponent({
      mode: CodeEditorMode.Markdown,
      lineNumbers: true
    });

    const textarea = getByTestId("textarea");
    expect(fromTextAreaSpy).toHaveBeenCalledTimes(1);
    expect(fromTextAreaSpy).toHaveBeenCalledWith(textarea, {
      mode: CodeEditorMode.Markdown,
      lineNumbers: true
    });
  });

  it("onChange is called with correct(default) value", () => {
    const onChangeMock = jest.fn();
    const someDefaultValue = "some default value";

    renderComponent({ onChange: onChangeMock, defaultValue: someDefaultValue });

    const codeMirrorInstance = fromTextAreaSpy.mock.results[0].value;
    const codeMirrorOnChange = codeMirrorInstance._handlers.change[0];

    // simulate change event call
    codeMirrorOnChange(codeMirrorInstance);

    expect(onChangeMock).toHaveBeenCalledTimes(1);
    expect(onChangeMock).toHaveBeenCalledWith(someDefaultValue);
  });

  it("defaultValue of rendered textarea is correct", () => {
    const defaultValue = "some default value";
    const { getByTestId } = renderComponent({ defaultValue });

    expect((getByTestId("textarea") as HTMLTextAreaElement).value).toBe(
      defaultValue
    );
  });
});
